@@ -4,35 +4,43 @@
  */
 "use strict";
 
-const { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
+const { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
 );
-const { ctypes } = ChromeUtils.import("resource://gre/modules/ctypes.jsm");
-const { FileUtils } = ChromeUtils.import(
-  "resource://gre/modules/FileUtils.jsm"
+const { ctypes } = ChromeUtils.importESModule(
+  "resource://gre/modules/ctypes.sys.mjs"
 );
-const { HttpServer } = ChromeUtils.import("resource://testing-common/httpd.js");
-const { MockRegistrar } = ChromeUtils.import(
-  "resource://testing-common/MockRegistrar.jsm"
+const { FileUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/FileUtils.sys.mjs"
 );
-const { NetUtil } = ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
-const { Promise } = ChromeUtils.import("resource://gre/modules/Promise.jsm");
-const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-const { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+const { HttpServer } = ChromeUtils.importESModule(
+  "resource://testing-common/httpd.sys.mjs"
+);
+const { MockRegistrar } = ChromeUtils.importESModule(
+  "resource://testing-common/MockRegistrar.sys.mjs"
+);
+const { NetUtil } = ChromeUtils.importESModule(
+  "resource://gre/modules/NetUtil.sys.mjs"
+);
+const { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
 
-const { X509 } = ChromeUtils.import("resource://gre/modules/psm/X509.jsm");
+const { X509 } = ChromeUtils.importESModule(
+  "resource://gre/modules/psm/X509.sys.mjs"
+);
 
-const isDebugBuild = Cc["@mozilla.org/xpcom/debug;1"].getService(Ci.nsIDebug2)
-  .isDebugBuild;
+const gIsDebugBuild = Cc["@mozilla.org/xpcom/debug;1"].getService(
+  Ci.nsIDebug2
+).isDebugBuild;
 
 // The test EV roots are only enabled in debug builds as a security measure.
-const gEVExpected = isDebugBuild;
+const gEVExpected = gIsDebugBuild;
 
-const CLIENT_AUTH_FILE_NAME = "ClientAuthRememberList.txt";
-const SSS_STATE_FILE_NAME = "SiteSecurityServiceState.txt";
-const PRELOAD_STATE_FILE_NAME = "SecurityPreloadState.txt";
+const CLIENT_AUTH_FILE_NAME = "ClientAuthRememberList.bin";
+const SSS_STATE_FILE_NAME = "SiteSecurityServiceState.bin";
+const SSS_STATE_OLD_FILE_NAME = "SiteSecurityServiceState.txt";
+const CERT_OVERRIDE_FILE_NAME = "cert_override.txt";
 
 const SEC_ERROR_BASE = Ci.nsINSSErrorsService.NSS_SEC_ERROR_BASE;
 const SSL_ERROR_BASE = Ci.nsINSSErrorsService.NSS_SSL_ERROR_BASE;
@@ -103,24 +111,21 @@ const MOZILLA_PKIX_ERROR_ADDITIONAL_POLICY_CONSTRAINT_FAILED =
   MOZILLA_PKIX_ERROR_BASE + 13;
 const MOZILLA_PKIX_ERROR_SELF_SIGNED_CERT = MOZILLA_PKIX_ERROR_BASE + 14;
 const MOZILLA_PKIX_ERROR_MITM_DETECTED = MOZILLA_PKIX_ERROR_BASE + 15;
-
-// Supported Certificate Usages
-const certificateUsageSSLClient = 0x0001;
-const certificateUsageSSLServer = 0x0002;
-const certificateUsageSSLCA = 0x0008;
-const certificateUsageEmailSigner = 0x0010;
-const certificateUsageEmailRecipient = 0x0020;
+const MOZILLA_PKIX_ERROR_INSUFFICIENT_CERTIFICATE_TRANSPARENCY =
+  MOZILLA_PKIX_ERROR_BASE + 16;
+const MOZILLA_PKIX_ERROR_ISSUER_NO_LONGER_TRUSTED =
+  MOZILLA_PKIX_ERROR_BASE + 17;
 
 // A map from the name of a certificate usage to the value of the usage.
 // Useful for printing debugging information and for enumerating all supported
 // usages.
-const allCertificateUsages = {
-  certificateUsageSSLClient,
-  certificateUsageSSLServer,
-  certificateUsageSSLCA,
-  certificateUsageEmailSigner,
-  certificateUsageEmailRecipient,
-};
+const verifyUsages = new Map([
+  ["verifyUsageTLSClient", Ci.nsIX509CertDB.verifyUsageTLSClient],
+  ["verifyUsageTLSServer", Ci.nsIX509CertDB.verifyUsageTLSServer],
+  ["verifyUsageTLSServerCA", Ci.nsIX509CertDB.verifyUsageTLSServerCA],
+  ["verifyUsageEmailSigner", Ci.nsIX509CertDB.verifyUsageEmailSigner],
+  ["verifyUsageEmailRecipient", Ci.nsIX509CertDB.verifyUsageEmailRecipient],
+]);
 
 const NO_FLAGS = 0;
 
@@ -160,20 +165,34 @@ function arrayToString(a) {
 // PEM to the format that nsIX509CertDB requires.
 function pemToBase64(pem) {
   return pem
-    .replace(/-----BEGIN CERTIFICATE-----/, "")
-    .replace(/-----END CERTIFICATE-----/, "")
+    .replace(/-----BEGIN (CERTIFICATE|(EC )?PRIVATE KEY)-----/, "")
+    .replace(/-----END (CERTIFICATE|(EC )?PRIVATE KEY)-----/, "")
     .replace(/[\r\n]/g, "");
 }
 
 function build_cert_chain(certNames, testDirectory = "bad_certs") {
   let certList = [];
-  certNames.forEach(function(certName) {
+  certNames.forEach(function (certName) {
     let cert = constructCertFromFile(`${testDirectory}/${certName}.pem`);
     certList.push(cert);
   });
   return certList;
 }
 
+function areCertsEqual(certA, certB) {
+  let derA = certA.getRawDER();
+  let derB = certB.getRawDER();
+  if (derA.length != derB.length) {
+    return false;
+  }
+  for (let i = 0; i < derA.length; i++) {
+    if (derA[i] != derB[i]) {
+      return false;
+    }
+  }
+  return true;
+}
+
 function areCertArraysEqual(certArrayA, certArrayB) {
   if (certArrayA.length != certArrayB.length) {
     return false;
@@ -182,7 +201,7 @@ function areCertArraysEqual(certArrayA, certArrayB) {
   for (let i = 0; i < certArrayA.length; i++) {
     const certA = certArrayA[i];
     const certB = certArrayB[i];
-    if (!certA.equals(certB)) {
+    if (!areCertsEqual(certA, certB)) {
       return false;
     }
   }
@@ -201,6 +220,17 @@ function readFile(file) {
   return data;
 }
 
+function readBinaryFile(file) {
+  let fstream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(
+    Ci.nsIFileInputStream
+  );
+  fstream.init(file, -1, 0, 0);
+  let available = fstream.available();
+  let bytes = NetUtil.readInputStream(fstream, available);
+  fstream.close();
+  return new Uint8Array(bytes);
+}
+
 function addCertFromFile(certdb, filename, trustString) {
   let certFile = do_get_file(filename, false);
   let certBytes = readFile(certFile);
@@ -275,16 +305,25 @@ function checkCertErrorGenericAtTime(
   time,
   /* optional */ isEVExpected,
   /* optional */ hostname,
-  /* optional */ flags = NO_FLAGS
+  /* optional */ flags = NO_FLAGS,
+  /* optional */ sctsFromTls = []
 ) {
-  return new Promise((resolve, reject) => {
+  return new Promise(resolve => {
     let result = new CertVerificationExpectedErrorResult(
       cert.commonName,
       expectedError,
       isEVExpected,
       resolve
     );
-    certdb.asyncVerifyCertAtTime(cert, usage, flags, hostname, time, result);
+    certdb.asyncVerifyCertAtTime(
+      cert,
+      usage,
+      flags,
+      hostname,
+      time,
+      sctsFromTls,
+      result
+    );
   });
 }
 
@@ -310,6 +349,56 @@ function checkCertErrorGeneric(
   );
 }
 
+// Helper for checkRootOfBuiltChain
+class CertVerificationExpectedRootResult {
+  constructor(certName, rootSha256SpkiDigest, resolve) {
+    this.certName = certName;
+    this.rootSha256SpkiDigest = rootSha256SpkiDigest;
+    this.resolve = resolve;
+  }
+
+  verifyCertFinished(aPRErrorCode, aVerifiedChain, _aHasEVPolicy) {
+    equal(
+      aPRErrorCode,
+      PRErrorCodeSuccess,
+      `verifying ${this.certName}: should succeed`
+    );
+    equal(
+      aVerifiedChain[aVerifiedChain.length - 1]
+        .sha256SubjectPublicKeyInfoDigest,
+      this.rootSha256SpkiDigest,
+      `verifying ${this.certName}: should build chain to ${this.rootSha256SpkiDigest}`
+    );
+    this.resolve();
+  }
+}
+
+function checkRootOfBuiltChain(
+  certdb,
+  cert,
+  rootSha256SpkiDigest,
+  time,
+  /* optional */ hostname,
+  /* optional */ flags = NO_FLAGS
+) {
+  return new Promise(resolve => {
+    let result = new CertVerificationExpectedRootResult(
+      cert.commonName,
+      rootSha256SpkiDigest,
+      resolve
+    );
+    certdb.asyncVerifyCertAtTime(
+      cert,
+      Ci.nsIX509CertDB.verifyUsageTLSServer,
+      flags,
+      hostname,
+      time,
+      [],
+      result
+    );
+  });
+}
+
 function checkEVStatus(certDB, cert, usage, isEVExpected) {
   return checkCertErrorGeneric(
     certDB,
@@ -449,7 +538,7 @@ function run_test() {
 */
 
 function add_tls_server_setup(serverBinName, certsPath, addDefaultRoot = true) {
-  add_test(function() {
+  add_test(function () {
     _setupTLSServerTest(serverBinName, certsPath, addDefaultRoot);
   });
 }
@@ -457,7 +546,7 @@ function add_tls_server_setup(serverBinName, certsPath, addDefaultRoot = true) {
 /**
  * Add a TLS connection test case.
  *
- * @param {String} aHost
+ * @param {string} aHost
  *   The hostname to pass in the SNI TLS extension; this should unambiguously
  *   identify which test is being run.
  * @param {PRErrorCode} aExpectedResult
@@ -491,7 +580,7 @@ function add_connection_test(
   /* optional */ aOriginAttributes,
   /* optional */ aEchConfig
 ) {
-  add_test(function() {
+  add_test(function () {
     if (aBeforeConnect) {
       aBeforeConnect();
     }
@@ -519,11 +608,17 @@ async function asyncConnectTo(
   function Connection(host) {
     this.host = host;
     this.thread = Services.tm.currentThread;
-    this.defer = Promise.defer();
+    this.defer = Promise.withResolvers();
     let sts = Cc["@mozilla.org/network/socket-transport-service;1"].getService(
       Ci.nsISocketTransportService
     );
-    this.transport = sts.createTransport(["ssl"], host, REMOTE_PORT, null);
+    this.transport = sts.createTransport(
+      ["ssl"],
+      host,
+      REMOTE_PORT,
+      null,
+      null
+    );
     if (aEchConfig) {
       this.transport.setEchConfig(atob(aEchConfig));
     }
@@ -541,7 +636,7 @@ async function asyncConnectTo(
 
   Connection.prototype = {
     // nsITransportEventSink
-    onTransportStatus(aTransport, aStatus, aProgress, aProgressMax) {
+    onTransportStatus(aTransport, aStatus) {
       if (
         !this.connected &&
         aStatus == Ci.nsISocketTransport.STATUS_CONNECTED_TO
@@ -567,7 +662,7 @@ async function asyncConnectTo(
     },
 
     // nsIOutputStreamCallback
-    onOutputStreamReady(aStream) {
+    onOutputStreamReady() {
       if (aAfterStreamOpen) {
         aAfterStreamOpen(this.transport);
       }
@@ -595,7 +690,7 @@ async function asyncConnectTo(
     return connection.go();
   }
 
-  return connectTo(aHost).then(function(conn) {
+  return connectTo(aHost).then(async function (conn) {
     info("handling " + aHost);
     let expectedNSResult =
       aExpectedResult == PRErrorCodeSuccess
@@ -608,7 +703,7 @@ async function asyncConnectTo(
     );
     if (aWithSecurityInfo) {
       aWithSecurityInfo(
-        conn.transport.securityInfo.QueryInterface(Ci.nsITransportSecurityInfo)
+        await conn.transport.tlsSocketControl.asyncGetSecurityInfo()
       );
     }
   });
@@ -664,31 +759,28 @@ async function asyncStartTLSTestServer(
 
   const CALLBACK_PORT = 8444;
 
-  let envSvc = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
   let greBinDir = Services.dirsvc.get("GreBinD", Ci.nsIFile);
-  envSvc.set("DYLD_LIBRARY_PATH", greBinDir.path);
+  Services.env.set("DYLD_LIBRARY_PATH", greBinDir.path);
   // TODO(bug 1107794): Android libraries are in /data/local/xpcb, but "GreBinD"
   // does not return this path on Android, so hard code it here.
-  envSvc.set("LD_LIBRARY_PATH", greBinDir.path + ":/data/local/xpcb");
-  envSvc.set("MOZ_TLS_SERVER_DEBUG_LEVEL", "3");
-  envSvc.set("MOZ_TLS_SERVER_CALLBACK_PORT", CALLBACK_PORT);
+  Services.env.set("LD_LIBRARY_PATH", greBinDir.path + ":/data/local/xpcb");
+  Services.env.set("MOZ_TLS_SERVER_DEBUG_LEVEL", "3");
+  Services.env.set("MOZ_TLS_SERVER_CALLBACK_PORT", CALLBACK_PORT);
 
   let httpServer = new HttpServer();
   let serverReady = new Promise(resolve => {
-    httpServer.registerPathHandler("/", function handleServerCallback(
-      aRequest,
-      aResponse
-    ) {
-      aResponse.setStatusLine(aRequest.httpVersion, 200, "OK");
-      aResponse.setHeader("Content-Type", "text/plain");
-      let responseBody = "OK!";
-      aResponse.bodyOutputStream.write(responseBody, responseBody.length);
-      executeSoon(function() {
-        httpServer.stop(resolve);
-      });
-    });
+    httpServer.registerPathHandler(
+      "/",
+      function handleServerCallback(aRequest, aResponse) {
+        aResponse.setStatusLine(aRequest.httpVersion, 200, "OK");
+        aResponse.setHeader("Content-Type", "text/plain");
+        let responseBody = "OK!";
+        aResponse.bodyOutputStream.write(responseBody, responseBody.length);
+        executeSoon(function () {
+          httpServer.stop(resolve);
+        });
+      }
+    );
     httpServer.start(CALLBACK_PORT);
   });
 
@@ -701,7 +793,7 @@ async function asyncStartTLSTestServer(
   // Using "sql:" causes the SQL DB to be used so we can run tests on Android.
   process.run(false, ["sql:" + certDir.path, Services.appinfo.processID], 2);
 
-  registerCleanupFunction(function() {
+  registerCleanupFunction(function () {
     process.kill();
   });
 
@@ -748,11 +840,11 @@ function generateOCSPResponses(ocspRespArray, nssDBlocation) {
 // serverIdentities.
 function getFailingHttpServer(serverPort, serverIdentities) {
   let httpServer = new HttpServer();
-  httpServer.registerPrefixHandler("/", function(request, response) {
+  httpServer.registerPrefixHandler("/", function () {
     Assert.ok(false, "HTTP responder should not have been queried");
   });
   httpServer.identity.setPrimary("http", serverIdentities.shift(), serverPort);
-  serverIdentities.forEach(function(identity) {
+  serverIdentities.forEach(function (identity) {
     httpServer.identity.add("http", identity, serverPort);
   });
   httpServer.start(serverPort);
@@ -792,51 +884,52 @@ function startOCSPResponder(
   expectedResponseTypes,
   responseHeaderPairs = []
 ) {
-  let ocspResponseGenerationArgs = expectedCertNames.map(function(
-    expectedNick
-  ) {
-    let responseType = "good";
-    if (expectedResponseTypes && expectedResponseTypes.length >= 1) {
-      responseType = expectedResponseTypes.shift();
+  let ocspResponseGenerationArgs = expectedCertNames.map(
+    function (expectedNick) {
+      let responseType = "good";
+      if (expectedResponseTypes && expectedResponseTypes.length >= 1) {
+        responseType = expectedResponseTypes.shift();
+      }
+      return [responseType, expectedNick, "unused", 0];
     }
-    return [responseType, expectedNick, "unused", 0];
-  });
+  );
   let ocspResponses = generateOCSPResponses(
     ocspResponseGenerationArgs,
     nssDBLocation
   );
   let httpServer = new HttpServer();
-  httpServer.registerPrefixHandler("/", function handleServerCallback(
-    aRequest,
-    aResponse
-  ) {
-    info("got request for: " + aRequest.path);
-    let basePath = aRequest.path.slice(1).split("/")[0];
-    if (expectedBasePaths.length >= 1) {
-      Assert.equal(
-        basePath,
-        expectedBasePaths.shift(),
-        "Actual and expected base path should match"
+  httpServer.registerPrefixHandler(
+    "/",
+    function handleServerCallback(aRequest, aResponse) {
+      info("got request for: " + aRequest.path);
+      let basePath = aRequest.path.slice(1).split("/")[0];
+      if (expectedBasePaths.length >= 1) {
+        Assert.equal(
+          basePath,
+          expectedBasePaths.shift(),
+          "Actual and expected base path should match"
+        );
+      }
+      Assert.greaterOrEqual(
+        expectedCertNames.length,
+        1,
+        "expectedCertNames should contain >= 1 entries"
       );
+      if (expectedMethods && expectedMethods.length >= 1) {
+        Assert.equal(
+          aRequest.method,
+          expectedMethods.shift(),
+          "Actual and expected fetch method should match"
+        );
+      }
+      aResponse.setStatusLine(aRequest.httpVersion, 200, "OK");
+      aResponse.setHeader("Content-Type", "application/ocsp-response");
+      for (let headerPair of responseHeaderPairs) {
+        aResponse.setHeader(headerPair[0], headerPair[1]);
+      }
+      aResponse.write(ocspResponses.shift());
     }
-    Assert.ok(
-      expectedCertNames.length >= 1,
-      "expectedCertNames should contain >= 1 entries"
-    );
-    if (expectedMethods && expectedMethods.length >= 1) {
-      Assert.equal(
-        aRequest.method,
-        expectedMethods.shift(),
-        "Actual and expected fetch method should match"
-      );
-    }
-    aResponse.setStatusLine(aRequest.httpVersion, 200, "OK");
-    aResponse.setHeader("Content-Type", "application/ocsp-response");
-    for (let headerPair of responseHeaderPairs) {
-      aResponse.setHeader(headerPair[0], headerPair[1]);
-    }
-    aResponse.write(ocspResponses.shift());
-  });
+  );
   httpServer.identity.setPrimary("http", identity, serverPort);
   httpServer.start(serverPort);
   return {
@@ -876,7 +969,7 @@ function startOCSPResponder(
 // Given an OCSP responder (see startOCSPResponder), returns a promise that
 // resolves when the responder has successfully stopped.
 function stopOCSPResponder(responder) {
-  return new Promise((resolve, reject) => {
+  return new Promise(resolve => {
     responder.stop(resolve);
   });
 }
@@ -885,51 +978,23 @@ function stopOCSPResponder(responder) {
 
 // Helper function for add_cert_override_test. Probably doesn't need to be
 // called directly.
-function add_cert_override(aHost, aExpectedBits, aSecurityInfo) {
-  let bits =
-    (aSecurityInfo.isUntrusted
-      ? Ci.nsICertOverrideService.ERROR_UNTRUSTED
-      : 0) |
-    (aSecurityInfo.isDomainMismatch
-      ? Ci.nsICertOverrideService.ERROR_MISMATCH
-      : 0) |
-    (aSecurityInfo.isNotValidAtThisTime
-      ? Ci.nsICertOverrideService.ERROR_TIME
-      : 0);
-
-  Assert.equal(
-    bits,
-    aExpectedBits,
-    "Actual and expected override bits should match"
-  );
+function add_cert_override(aHost, aSecurityInfo) {
   let cert = aSecurityInfo.serverCert;
   let certOverrideService = Cc[
     "@mozilla.org/security/certoverride;1"
   ].getService(Ci.nsICertOverrideService);
-  certOverrideService.rememberValidityOverride(
-    aHost,
-    8443,
-    cert,
-    aExpectedBits,
-    true
-  );
+  certOverrideService.rememberValidityOverride(aHost, 8443, {}, cert, true);
 }
 
-// Given a host, expected error bits (see nsICertOverrideService.idl), and an
-// expected error code, tests that an initial connection to the host fails
-// with the expected errors and that adding an override results in a subsequent
-// connection succeeding.
-function add_cert_override_test(
-  aHost,
-  aExpectedBits,
-  aExpectedError,
-  aExpectedSecInfo = undefined
-) {
+// Given a host and an expected error code, tests that an initial connection to
+// the host fails with the expected error and that adding an override results
+// in a subsequent connection succeeding.
+function add_cert_override_test(aHost, aExpectedError) {
   add_connection_test(
     aHost,
     aExpectedError,
     null,
-    add_cert_override.bind(this, aHost, aExpectedBits)
+    add_cert_override.bind(this, aHost)
   );
   add_connection_test(aHost, PRErrorCodeSuccess, null, aSecurityInfo => {
     Assert.ok(
@@ -937,13 +1002,6 @@ function add_cert_override_test(
         Ci.nsIWebProgressListener.STATE_CERT_USER_OVERRIDDEN,
       "Cert override flag should be set on the security state"
     );
-    if (aExpectedSecInfo) {
-      if (aExpectedSecInfo.failedCertChain) {
-        ok(
-          aExpectedSecInfo.failedCertChain.equals(aSecurityInfo.failedCertChain)
-        );
-      }
-    }
   });
 }
 
@@ -951,53 +1009,28 @@ function add_cert_override_test(
 // add_cert_override except it may not be the case that the connection has an
 // SecInfo set on it. In this case, the error was not overridable anyway, so
 // we consider it a success.
-function attempt_adding_cert_override(aHost, aExpectedBits, aSecurityInfo) {
+function attempt_adding_cert_override(aHost, aSecurityInfo) {
   if (aSecurityInfo.serverCert) {
-    let bits =
-      (aSecurityInfo.isUntrusted
-        ? Ci.nsICertOverrideService.ERROR_UNTRUSTED
-        : 0) |
-      (aSecurityInfo.isDomainMismatch
-        ? Ci.nsICertOverrideService.ERROR_MISMATCH
-        : 0) |
-      (aSecurityInfo.isNotValidAtThisTime
-        ? Ci.nsICertOverrideService.ERROR_TIME
-        : 0);
-    Assert.equal(
-      bits,
-      aExpectedBits,
-      "Actual and expected override bits should match"
-    );
     let cert = aSecurityInfo.serverCert;
     let certOverrideService = Cc[
       "@mozilla.org/security/certoverride;1"
     ].getService(Ci.nsICertOverrideService);
-    certOverrideService.rememberValidityOverride(
-      aHost,
-      8443,
-      cert,
-      aExpectedBits,
-      true
-    );
+    certOverrideService.rememberValidityOverride(aHost, 8443, {}, cert, true);
   }
 }
 
-// Given a host, expected error bits (see nsICertOverrideService.idl), and
-// an expected error code, tests that an initial connection to the host fails
-// with the expected errors and that adding an override does not result in a
-// subsequent connection succeeding (i.e. the same error code is encountered).
+// Given a host and an expected error code, tests that an initial connection to
+// the host fails with the expected error and that adding an override does not
+// result in a subsequent connection succeeding (i.e. the same error code is
+// encountered).
 // The idea here is that for HSTS hosts or hosts with key pins, no error is
 // overridable, even if an entry is added to the override service.
-function add_prevented_cert_override_test(
-  aHost,
-  aExpectedBits,
-  aExpectedError
-) {
+function add_prevented_cert_override_test(aHost, aExpectedError) {
   add_connection_test(
     aHost,
     aExpectedError,
     null,
-    attempt_adding_cert_override.bind(this, aHost, aExpectedBits)
+    attempt_adding_cert_override.bind(this, aHost)
   );
   add_connection_test(aHost, aExpectedError);
 }
@@ -1011,7 +1044,7 @@ class CertVerificationResult {
     this.resolve = resolve;
   }
 
-  verifyCertFinished(aPRErrorCode, aVerifiedChain, aHasEVPolicy) {
+  verifyCertFinished(aPRErrorCode) {
     if (this.successExpected) {
       equal(
         aPRErrorCode,
@@ -1040,19 +1073,19 @@ class CertVerificationResult {
  *   The certificate database to use to verify the certificate.
  * @param {nsIX509Cert} cert
  *   The certificate to be verified.
- * @param {Number[]} expectedUsages
+ * @param {number[]} expectedUsages
  *   A list of usages (as their integer values) that are expected to verify
  *   successfully.
- * @return {Promise}
+ * @returns {Promise}
  *   A promise that will resolve with no value when all asynchronous operations
  *   have completed.
  */
 function asyncTestCertificateUsages(certdb, cert, expectedUsages) {
   let now = new Date().getTime() / 1000;
   let promises = [];
-  Object.keys(allCertificateUsages).forEach(usageString => {
-    let promise = new Promise((resolve, reject) => {
-      let usage = allCertificateUsages[usageString];
+  verifyUsages.keys().forEach(usageString => {
+    let promise = new Promise(resolve => {
+      let usage = verifyUsages.get(usageString);
       let successExpected = expectedUsages.includes(usage);
       let result = new CertVerificationResult(
         cert.commonName,
@@ -1061,7 +1094,7 @@ function asyncTestCertificateUsages(certdb, cert, expectedUsages) {
         resolve
       );
       let flags = Ci.nsIX509CertDB.FLAG_LOCAL_ONLY;
-      certdb.asyncVerifyCertAtTime(cert, usage, flags, null, now, result);
+      certdb.asyncVerifyCertAtTime(cert, usage, flags, null, now, [], result);
     });
     promises.push(promise);
   });
@@ -1075,9 +1108,9 @@ function asyncTestCertificateUsages(certdb, cert, expectedUsages) {
  * @param {nsIFile} libraryFile
  *                  The dynamic library file that implements the module to
  *                  load.
- * @param {String} moduleName
+ * @param {string} moduleName
  *                 What to call the module.
- * @param {Boolean} expectModuleUnloadToFail
+ * @param {boolean} expectModuleUnloadToFail
  *                  Should be set to true for tests that manually unload the
  *                  test module, so the attempt to auto unload the test module
  *                  doesn't cause a test failure. Should be set to false
@@ -1104,8 +1137,8 @@ function loadPKCS11Module(libraryFile, moduleName, expectModuleUnloadToFail) {
 }
 
 /**
- * @param {String} data
- * @returns {String}
+ * @param {string} data
+ * @returns {string}
  */
 function hexify(data) {
   // |slice(-2)| chomps off the last two characters of a string.
@@ -1118,7 +1151,7 @@ function hexify(data) {
 }
 
 /**
- * @param {String[]} lines
+ * @param {string[]} lines
  *        Lines to write. Each line automatically has "\n" appended to it when
  *        being written.
  * @param {nsIFileOutputStream} outputStream
@@ -1132,13 +1165,13 @@ function writeLinesAndClose(lines, outputStream) {
 }
 
 /**
- * @param {String} moduleName
+ * @param {string} moduleName
  *        The name of the module that should not be loaded.
- * @param {String} libraryName
+ * @param {string} libraryName
  *        A unique substring of name of the dynamic library file of the module
  *        that should not be loaded.
  */
-function checkPKCS11ModuleNotPresent(moduleName, libraryName) {
+function checkPKCS11ModuleNotPresent(moduleName, libraryName = "undefined") {
   let moduleDB = Cc["@mozilla.org/security/pkcs11moduledb;1"].getService(
     Ci.nsIPKCS11ModuleDB
   );
@@ -1153,10 +1186,12 @@ function checkPKCS11ModuleNotPresent(moduleName, libraryName) {
       moduleName,
       `Non-test module name shouldn't equal '${moduleName}'`
     );
-    ok(
-      !(module.libName && module.libName.includes(libraryName)),
-      `Non-test module lib name should not include '${libraryName}'`
-    );
+    if (libraryName != "undefined") {
+      ok(
+        !(module.libName && module.libName.includes(libraryName)),
+        `Non-test module lib name should not include '${libraryName}'`
+      );
+    }
   }
 }
 
@@ -1164,15 +1199,15 @@ function checkPKCS11ModuleNotPresent(moduleName, libraryName) {
  * Checks that the test module exists in the module list.
  * Also checks various attributes of the test module for correctness.
  *
- * @param {String} moduleName
+ * @param {string} moduleName
  *                 The name of the module that should be present.
- * @param {String} libraryName
+ * @param {string} libraryName
  *                 A unique substring of the name of the dynamic library file
  *                 of the module that should be loaded.
  * @returns {nsIPKCS11Module}
  *          The test module.
  */
-function checkPKCS11ModuleExists(moduleName, libraryName) {
+function checkPKCS11ModuleExists(moduleName, libraryName = "undefined") {
   let moduleDB = Cc["@mozilla.org/security/pkcs11moduledb;1"].getService(
     Ci.nsIPKCS11ModuleDB
   );
@@ -1189,11 +1224,17 @@ function checkPKCS11ModuleExists(moduleName, libraryName) {
     }
   }
   notEqual(testModule, null, "Test module should have been found");
-  notEqual(testModule.libName, null, "Test module lib name should not be null");
-  ok(
-    testModule.libName.includes(ctypes.libraryName(libraryName)),
-    `Test module lib name should include lib name of '${libraryName}'`
-  );
+  if (libraryName != "undefined") {
+    notEqual(
+      testModule.libName,
+      null,
+      "Test module lib name should not be null"
+    );
+    ok(
+      testModule.libName.includes(ctypes.libraryName(libraryName)),
+      `Test module lib name should include lib name of '${libraryName}'`
+    );
+  }
 
   return testModule;
 }
@@ -1211,14 +1252,11 @@ function getSubjectAndSPKIHash(nsCert) {
 }
 
 function run_certutil_on_directory(directory, args, expectSuccess = true) {
-  let envSvc = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
   let greBinDir = Services.dirsvc.get("GreBinD", Ci.nsIFile);
-  envSvc.set("DYLD_LIBRARY_PATH", greBinDir.path);
+  Services.env.set("DYLD_LIBRARY_PATH", greBinDir.path);
   // TODO(bug 1107794): Android libraries are in /data/local/xpcb, but "GreBinD"
   // does not return this path on Android, so hard code it here.
-  envSvc.set("LD_LIBRARY_PATH", greBinDir.path + ":/data/local/xpcb");
+  Services.env.set("LD_LIBRARY_PATH", greBinDir.path + ":/data/local/xpcb");
   let certutilBin = _getBinaryUtil("certutil");
   let process = Cc["@mozilla.org/process/util;1"].createInstance(Ci.nsIProcess);
   process.init(certutilBin);
@@ -1229,3 +1267,105 @@ function run_certutil_on_directory(directory, args, expectSuccess = true) {
     Assert.equal(process.exitValue, 0, "certutil should succeed");
   }
 }
+
+function get_data_storage_contents(dataStorageFileName) {
+  let stateFile = do_get_profile();
+  stateFile.append(dataStorageFileName);
+  if (!stateFile.exists()) {
+    return undefined;
+  }
+  return readFile(stateFile);
+}
+
+function u16_to_big_endian_bytes(u16) {
+  Assert.less(u16, 65536);
+  return [u16 / 256, u16 % 256];
+}
+
+// Appends a line to the given data storage file (as an nsIOutputStream).
+// score is an integer representing the number of unique days the item has been accessed.
+// lastAccessed is the day since the epoch the item was last accessed.
+// key and value are strings representing the key and value of the item.
+function append_line_to_data_storage_file(
+  outputStream,
+  score,
+  lastAccessed,
+  key,
+  value,
+  valueLength = 24,
+  useBadChecksum = false
+) {
+  let line = arrayToString(u16_to_big_endian_bytes(score));
+  line = line + arrayToString(u16_to_big_endian_bytes(lastAccessed));
+  line = line + key;
+  let keyPadding = [];
+  for (let i = 0; i < 256 - key.length; i++) {
+    keyPadding.push(0);
+  }
+  line = line + arrayToString(keyPadding);
+  line = line + value;
+  let valuePadding = [];
+  for (let i = 0; i < valueLength - value.length; i++) {
+    valuePadding.push(0);
+  }
+  line = line + arrayToString(valuePadding);
+  let checksum = 0;
+  Assert.equal(line.length % 2, 0);
+  for (let i = 0; i < line.length; i += 2) {
+    checksum ^= (line.charCodeAt(i) << 8) + line.charCodeAt(i + 1);
+  }
+  line =
+    arrayToString(
+      u16_to_big_endian_bytes(useBadChecksum ? ~checksum & 0xffff : checksum)
+    ) + line;
+  outputStream.write(line, line.length);
+}
+
+// Helper constants for setting security.pki.certificate_transparency.mode.
+const CT_MODE_DISABLE = 0;
+const CT_MODE_COLLECT_TELEMETRY = 1;
+const CT_MODE_ENFORCE = 2;
+
+// Helper function for add_ct_test. Returns a function that checks that the
+// nsITransportSecurityInfo of the connection has the expected CT and resumed
+// statuses.
+function expectCT(expectedCTValue, expectedResumed) {
+  return securityInfo => {
+    Assert.equal(
+      securityInfo.certificateTransparencyStatus,
+      expectedCTValue,
+      "actual and expected CT status should match"
+    );
+    Assert.equal(
+      securityInfo.resumed,
+      expectedResumed,
+      "connection should be resumed (or not) as expected"
+    );
+  };
+}
+
+// Helper function to add a certificate transparency test. The connection is
+// expected to succeed with the given CT status (see nsITransportSecurityInfo).
+// Additionally, if an additional connection is made, it is expected that TLS
+// resumption is used and that the CT status is the same with the resumed
+// connection.
+function add_ct_test(host, expectedCTValue, expectConnectionSuccess) {
+  add_connection_test(
+    host,
+    expectConnectionSuccess
+      ? PRErrorCodeSuccess
+      : MOZILLA_PKIX_ERROR_INSUFFICIENT_CERTIFICATE_TRANSPARENCY,
+    null,
+    expectCT(expectedCTValue, false)
+  );
+  // Test that session resumption results in the same expected CT status for
+  // successful connections.
+  if (expectConnectionSuccess) {
+    add_connection_test(
+      host,
+      PRErrorCodeSuccess,
+      null,
+      expectCT(expectedCTValue, true)
+    );
+  }
+}
